/* blob.ec
 * This sample program is mainly concerned about BLOB related ESQL.
 * Including:
 *     1. put blob.
 *     2. get blob.
 */

#include <stdio.h>
#include <string.h>

EXEC SQL include sqlca;
EXEC SQL include dbenvca;
EXEC SQL include sqlda;
#define STRING_LEN 1024
#define MAX_BUF_SIZE 2048

#define chkErr() \
        { \
        if (SQLCODE && SQLCODE != SQL_NO_DATA_FOUND) \
           { \
           if (SQLCODE == SQL_SUCCESS_WITH_INFO) \
              printf("WARN(%d):%s\n", sqlca.sqlerrd[0], sqlca.sqlerrmc); \
           else \
              { \
              printf("ERR(%d):%s\n", sqlca.sqlerrd[0], sqlca.sqlerrmc); \
              } \
           } \
        }

void demo_put_blob(int index, char *fname);
void demo_get_blob();

int main()
{
   EXEC SQL begin declare section;
   char user[8], passwd[8];           /* connection information         */
   char dbname[18];                   /* char type is fix length string */
   EXEC SQL end declare section;

   strcpy(dbname, "DBSAMPLE4");          /* get db,user,password info */
   strcpy(user, "SYSADM");
   strcpy(passwd, "");
 
   /* you can also directly type EXEC SQL connect to db_name user_id passwd */
   EXEC SQL connect to :dbname :user :passwd;

   printf("This is a sample program which inserts blob file 'logo.bmp' into BMP\n");
   printf("column of Card table OF DBSAMPLE4 database.\n");
   printf("We then retrive all the blob out and save them as bitmap files named \n");
   printf("from BMP1.BMP to BMP8.BMP.\n");
   printf("<<< demo_put_blob >>>\n");
   demo_put_blob(100, "logo.bmp");
   printf("\n");
   printf("<<< demo_get_blob >>>\n");
   demo_get_blob();
   EXEC SQL delete from Card where Num = 100;
   EXEC SQL commit work;

   EXEC SQL disconnect;
   if (SQLCODE)
      printf("ERR(%d):%s\n", sqlca.sqlerrd[0], sqlca.sqlerrmc);    
   return 0;
}

void demo_put_blob(int index, char * fname)
{
   EXEC SQL begin declare section;
      int Num;
      longvarbinary BMP;
   EXEC SQL end declare section;
   FILE *stream;
   int numread;
   long count=0, delay;
   char buf[MAX_BUF_SIZE];

   if( (stream  = fopen( fname, "rb" )) == NULL )
   {
      printf( "The file '%s' was not opened\n", fname );   
      exit(0);
   }

   /* Prepare an ESQL statement and declare the BLOB host variable as 
      a question mark, to indicate that this BLOB host variable will 
      be bound later                                                  */
   EXEC SQL prepare stmt from "INSERT INTO Card (Num, BMP) VALUES (:Num, ?)";
   chkErr();
   Num=index;
   BMP.buf = buf;
   BMP.bufsize = STRING_LEN;      

   /* Execute this ESQL statement */
   EXEC SQL execute stmt;
   chkErr();

   /* Declare when to begin to put the BLOB */
   EXEC SQL begin put blob for stmt;     
   chkErr();
   do {
      numread = fread( buf, sizeof( char ), STRING_LEN, stream );
      BMP.buf = buf;
      BMP.bufsize=numread;      
      /* Define which BLOB variable to use for putting the BLOB and filling 
         the information of the bufsize and bufptr. */
      EXEC SQL put blob for stmt using :BMP;
      chkErr();
      count += numread;
      printf("\r%ld         ", count);
      fflush(stdout);
      for (delay=0; delay<100000; delay++)
        fflush(stdout);
   } while (STRING_LEN == numread);

   /* Declare PUT BLOB operation is finished. */
   EXEC SQL end put blob for stmt;
   chkErr();
   printf("done.         \n");

   if( fclose( stream ) )
      printf( "The file '%s' was not closed\n", fname );
}

void demo_get_blob()
{
   EXEC SQL begin declare section;
      int Num;
      longvarbinary BMP;
      int i_BMP;
      int nCol;      
   EXEC SQL end declare section;
   char fname[20];
   FILE *stream;
   int numwritten;
   char buf[MAX_BUF_SIZE];

   /* Prepare an ESQL statement and declare the BLOB host variable as 
      a question mark. */
   EXEC SQL prepare stmt from "select Num, BMP from Card into :Num, ?";
   chkErr();
   
   /* Declare a cursor for a prepared statement */
   EXEC SQL declare demo_cursor cursor for stmt;
   chkErr();

   /* Open the cursor */
   EXEC SQL open demo_cursor;
   chkErr();
   nCol = 2; 
   BMP.bufsize = MAX_BUF_SIZE;
   BMP.buf = buf;

   while (1)
   {
     /* Fetch the cursor */
     EXEC SQL fetch demo_cursor into :Num, ?;
     chkErr();
     if (SQLCODE)
     {
       if (SQLWARN0 != 'W')
         break;
     }

     printf("Num = %d\n", Num);

     sprintf(fname, "BMP%d.BMP", Num);
     if( (stream  = fopen( fname, "w+b" )) == NULL )
       printf( "The file '%s' was not opened\n", fname );   

     while(1)
     {
       /* Define the column number in the BLOB host variable and the 
          available buffer size and valid buffer pointer */
       EXEC SQL get blob column :nCol for stmt using :BMP :i_BMP;
       if (SQLCODE == SQL_NO_DATA_FOUND)
         break;
       else 
         numwritten = fwrite( BMP.buf, sizeof( char ), (i_BMP<MAX_BUF_SIZE)?i_BMP: MAX_BUF_SIZE, stream );
     }

     if( fclose( stream ) )
       printf( "The file '%s' was not closed\n", fname );
   }
   /* Close the curosr. */
   EXEC SQL close demo_cursor;
   chkErr();
}
